package com.heytap.push.demo;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.RectF;
import android.util.AttributeSet;
import android.widget.Button;

import com.carlos.notificatoinbutton.library.R.styleable;

public class MyNotificationButton extends Button {
    private int notificationNumber;
    private float redCircleSize = 20.0F;
    private Paint paint;
    private RectF rectF;
    private boolean isChange;
    private int textColor;
    private int circleColor;

    public MyNotificationButton(Context context) {
        super(context);
        this.paint = new Paint();
        this.paint.setAntiAlias(true);
    }

    public MyNotificationButton(Context context, AttributeSet attrs) {
        super(context, attrs);
        this.initProperty(context, attrs);
    }

    public MyNotificationButton(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.initProperty(context, attrs);
    }

    private void initProperty(Context context, AttributeSet attributeSet) {
        TypedArray typedArray = context.obtainStyledAttributes(attributeSet, styleable.notButton);
        this.redCircleSize = typedArray.getDimension(styleable.notButton_circleSize, 20.0F);
        this.circleColor = typedArray.getColor(styleable.notButton_circleBgColor, -65536);
        this.textColor = typedArray.getColor(styleable.notButton_textColor, -1);
        this.paint = new Paint();
        this.paint.setAntiAlias(true);
        typedArray.recycle();
    }

    public void draw(Canvas canvas) {
        canvas.save();
        canvas.scale(((float)this.getWidth() - this.redCircleSize) / (float)this.getWidth(), ((float)this.getHeight() - this.redCircleSize) / (float)this.getHeight());
        canvas.translate(0.0F, this.redCircleSize);
        super.draw(canvas);
        canvas.restore();
        this.drawRedCircle(canvas);
    }

    private void drawRedCircle(Canvas canvas) {
        if (this.notificationNumber > 0) {
            this.paint.setColor(this.circleColor);
            if (this.rectF == null || this.isChange) {
                if (this.notificationNumber < 10) {
                    this.rectF = new RectF((float)this.getWidth() - this.redCircleSize * 2.0F, 0.0F, (float)this.getWidth(), this.redCircleSize * 2.0F);
                } else if (this.notificationNumber < 100) {
                    this.rectF = new RectF((float)this.getWidth() - this.redCircleSize * 3.0F, 0.0F, (float)this.getWidth(), this.redCircleSize * 2.0F);
                } else {
                    this.rectF = new RectF((float)this.getWidth() - this.redCircleSize * 4.0F, 0.0F, (float)this.getWidth(), this.redCircleSize * 2.0F);
                }
            }

            canvas.drawRoundRect(this.rectF, this.redCircleSize, this.redCircleSize, this.paint);
            this.paint.setTextSize(0);
        }
    }

    public void setNotificationNumber(int notificationNumber) {
        if (notificationNumber != this.notificationNumber) {
            this.isChange = true;
        }

        this.notificationNumber = notificationNumber;
        this.invalidate();
    }
}
