package com.heytap.push.demo.component;

import android.annotation.NonNull;
import android.annotation.Nullable;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;

import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ScrollView;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentTransaction;

import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.heytap.msp.push.HeytapPushManager;
import com.heytap.msp.push.callback.ICallBackResultService;
import com.heytap.msp.push.callback.IGetAppNotificationCallBackService;
import com.heytap.msp.push.callback.ISetAppNotificationCallBackService;
import com.heytap.msp.push.constant.EventConstant;
import com.heytap.msp.push.mode.DataMessage;
import com.heytap.msp.push.mode.MessageStat;
import com.heytap.push.demo.R;
import com.heytap.push.demo.component.activity.AboutActivity;
import com.heytap.push.demo.component.fragment.BaseFragment;
import com.heytap.push.demo.component.fragment.PushConfigFragment;
import com.heytap.push.demo.component.fragment.PushInfoFragment;
import com.heytap.push.demo.component.fragment.UserBandingFragment;
import com.heytap.push.demo.constants.AppParam;
import com.heytap.push.demo.util.LogUtil;
import com.heytap.push.demo.util.NotificationUtil;
import com.heytap.push.demo.util.TestModeUtil;

import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import static com.heytap.push.demo.util.TestModeUtil.TYPE_LOG;

public class DemoActivity extends AppCompatActivity implements TestModeUtil.UpdateTestMode {

    /**
     * 显示的log格式
     */
    private static final String LOG_FORMAT = "time ->[tag] msg";
    private static final String LOG_FORMAT_NO_TAG = "time -> msg";
    private static final int MAX_LOG_SIZE = 3000;
    /**
     * log中的时间格式
     */
    private static SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:SS", Locale.CHINA);
    /**
     * log信息显示到界面上，方便调试
     */
    private TextView logInfoTextView;
    private ScrollView mScrollView;
    /**
     * 用户操作和反馈信息，显示到界面上方便调试
     */
    private List<String> logs = new CopyOnWriteArrayList<>();
    private BaseFragment userBandingFragment;
    private BaseFragment pushConfigFragment;
    private BaseFragment[] fragments;
    private PushInfoFragment baseInfoFragment;
    private int lastShowFragment = 0;
    private IGetAppNotificationCallBackService getAppNotificationCallBackService = new IGetAppNotificationCallBackService() {
        @Override
        public void onGetAppNotificationSwitch(int i, int i1) {
            showResult("获取应用开关状态", "code=" + i + ",status=" + i1);
        }
    };
    private ISetAppNotificationCallBackService setAppNotificationCallBackService = new ISetAppNotificationCallBackService() {
        @Override
        public void onSetAppNotificationSwitch(int i) {
            showResult("设置应用开关", "code=" + i);
        }
    };
    private BottomNavigationView.OnNavigationItemSelectedListener mOnNavigationItemSelectedListener
            = new BottomNavigationView.OnNavigationItemSelectedListener() {

        @Override
        public boolean onNavigationItemSelected(@NonNull MenuItem item) {
            switchFragment(lastShowFragment, lastShowFragment = item.getOrder());
            return true;
        }
    };

    private boolean flag = true;

    /************************************************************************************
     * ***************************callbacks from mcs************************************
     ***********************************************************************************/
    private ICallBackResultService mPushCallback = new ICallBackResultService() {
        @Override
        public void onRegister(int code, String s) {
            if (code == 0) {
                showResult("注册成功", "registerId:" + s);
            } else {
                showResult("注册失败", "code=" + code + ",msg=" + s);
            }
        }

        @Override
        public void onUnRegister(int code) {
            if (code == 0) {
                showResult("注销成功", "code=" + code);
            } else {
                showResult("注销失败", "code=" + code);
            }
        }

        @Override
        public void onGetPushStatus(final int code, int status) {
            if (code == 0 && status == 0) {
                showResult("Push状态正常", "code=" + code + ",status=" + status);
            } else {
                showResult("Push状态错误", "code=" + code + ",status=" + status);
            }
        }

        @Override
        public void onGetNotificationStatus(final int code, final int status) {
            if (code == 0 && status == 0) {
                showResult("通知状态正常", "code=" + code + ",status=" + status);
            } else {
                showResult("通知状态错误", "code=" + code + ",status=" + status);
            }
        }

        @Override
        public void onError(int i, String s) {
            showResult("onError", "onError code : " + i + "   message : " + s);
        }

        @Override
        public void onSetPushTime(final int code, final String s) {
            showResult("SetPushTime", "code=" + code + ",result:" + s);
        }

    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        //顶部的appid等应用信息
        TextView appInfo = (TextView) findViewById(R.id.tv_app_info);
        String info = "AppId :\t" + AppParam.appId + "\n" + "AppKey :\t" + AppParam.appKey;
        appInfo.setText(info);
        logInfoTextView = (TextView) findViewById(R.id.tv_log_msg);
        mScrollView = (ScrollView) findViewById(R.id.scrollView);
        //底部tab切换控件使用了BottomNavigationView
        BottomNavigationView navigation = (BottomNavigationView) findViewById(R.id.navigation);
        navigation.setOnNavigationItemSelectedListener(mOnNavigationItemSelectedListener);
        //初始化fragment
        baseInfoFragment = new PushInfoFragment();
        userBandingFragment = new UserBandingFragment();
        pushConfigFragment = new PushConfigFragment();
        fragments = new BaseFragment[]{baseInfoFragment, userBandingFragment, pushConfigFragment};
        getSupportFragmentManager().beginTransaction().add(R.id.content, baseInfoFragment).show(baseInfoFragment).commit();
        initData();
        createChannelWithNoRedBadge();
        createChannelWithRedBadge();

//        registerReceiver(new BroadcastReceiver() {
//
//            @Override
//            public void onReceive(Context context, Intent intent) {
//                String action = intent.getAction();
//                if(intent.getAction().equals(Intent.ACTION_LOCALE_CHANGED)) {
//                    LogUtil.d("intent:"+intent.toString());
//                    //当前系统语音被更改了
//                }
//            }
//        }, new IntentFilter(Intent.ACTION_LOCALE_CHANGED));
    }




    private void createChannelWithNoRedBadge() {
        NotificationManager notificationManager = NotificationUtil.getNotificationManager(this);
        if (notificationManager != null) {

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                NotificationChannel channel = new NotificationChannel("no_red_badge", "运营推广",
                        NotificationManager.IMPORTANCE_DEFAULT);
                channel.setShowBadge(false);
                notificationManager.createNotificationChannel(channel);

            }
        }
    }

    private void createChannelWithRedBadge() {
        NotificationManager notificationManager = NotificationUtil.getNotificationManager(this);
        if (notificationManager != null) {

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                NotificationChannel channel = new NotificationChannel("red_badge", "私信",
                        NotificationManager.IMPORTANCE_DEFAULT);
                channel.setShowBadge(true);
                notificationManager.createNotificationChannel(channel);
            }
        }
    }

    private void testData() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                while (flag){
                    HeytapPushManager.register(DemoActivity.this, AppParam.appKey, AppParam.appSecret, mPushCallback);//setPushCallback接口也可设置callback
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }).start();
    }

    public final static String SDK_APP_SECRET2 = "/fAHle8XOBBonC2GBjkKA5+s9Cni2GDDLbfJH6G0mliE7yKA8wnb43x5KkA=";
    private static final String IV_CONNECT = "%IV1%";
    private static final String IV = "1234abcddcba4321";
    public final static String SDK_APP_SECRET = "isvrbeT7qUywVEZ1Ia0/aUVA/TcFaeV0wC8qFLc8rg4=";
    private static final String TRANSFORMATION = "AES/CTR/NoPadding";
    private static final String ALGORITHM = "AES";
    /**
     * @param content
     * @return
     */
//    public static String encrypt( String content) throws Exception {
//        byte[] iv = IV.getBytes();
//        byte[] srcData = content.getBytes();
//        byte[] enData = WBkit.WBkit_AES_CTR_encrypt(iv, srcData, 0);
//        String ivStr = Base64.encodeBase64String(iv);
//        String enDataStr = Base64.encodeBase64String(enData);
//        return enDataStr +IV_CONNECT + ivStr;
//    }

//    /**
//     * @param secretKey aes对称秘钥
//     * @param content
//     * @return
//     */
//    public static String decrypt(String secretKey, String content) throws Exception {
//        String[] dataIvs = content.split(IV_CONNECT);
//        byte[] enData = Base64.decodeBase64(dataIvs[0]);
//        byte[] iv = Base64.decodeBase64(dataIvs[1]);
//        byte[] deData = WBkit.WBkit_AES_CTR_decrypt(iv, enData, 0);
//        return new String(deData);
//    }


    /**
     * @param content
     * @return
     */
    public static String decrypt( String content) throws Exception {
        String[] dataIvs = content.split(IV_CONNECT);
        byte[] enData = Base64.decodeBase64(dataIvs[0]);
        byte[] iv = Base64.decodeBase64(dataIvs[1]);
        SecretKeySpec secretKeySpec = new SecretKeySpec(Base64.decodeBase64(SDK_APP_SECRET), ALGORITHM);
        Cipher cipher = Cipher.getInstance(TRANSFORMATION);
        IvParameterSpec ivParameterSpec = new IvParameterSpec(iv);
        cipher.init(Cipher.DECRYPT_MODE, secretKeySpec, ivParameterSpec);
        byte[] deData = cipher.doFinal(enData);
        return new String(deData);
    }

    private void initData() {
        Handler handler = new Handler(Looper.getMainLooper());
        LogUtil.d("looperName",handler.getLooper().getThread().getName());
//        try {
//            String test1 = encrypt(SDK_APP_SECRET);
//            LogUtil.d("test1 : "+test1);
//            String test2 = decrypt(test1 );
//            LogUtil.d("test2 : "+test2);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
        //初始化push，调用注册接口
        addLog("AppId", AppParam.appId);
        addLog("appKey", AppParam.appKey);
        addLog("appSecret", AppParam.appSecret);
        addLog("pkgName", getPackageName());
        addLog("", "");
        TestModeUtil.setUpdateTestMode(this);
        onLogUpdate(TYPE_LOG);
        onLogUpdate(TestModeUtil.TYPE_STATUS);
        addLog("初始化注册", "调用register接口");
        HeytapPushManager.register(this, AppParam.appKey, AppParam.appSecret, mPushCallback);//setPushCallback接口也可设置callback
        HeytapPushManager.requestNotificationPermission();
        try {

        } catch (Exception e) {
            e.printStackTrace();
            TestModeUtil.addLogString(e.getLocalizedMessage());
        }
    }


    @Override
    public void onLogUpdate(int type) {
        if (type == TYPE_LOG) {
            logInfoTextView.post(new Runnable() {
                @Override
                public void run() {
                    String info = TestModeUtil.getLastLog();
                    if (info == null) return;
                    while (info != null) {
                        logInfoTextView.append(info + "\n");
                        info = TestModeUtil.getLastLog();
                    }
                    //滚动log到底部
                    mScrollView.fullScroll(ScrollView.FOCUS_DOWN);
                }
            });
        } else if (type == TestModeUtil.TYPE_STATUS) {
            LogUtil.w("LOG TYPE_STATUS");
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        onLogUpdate(TYPE_LOG);
        onLogUpdate(TestModeUtil.TYPE_STATUS);
        if (NotificationUtil.hasRedMessage(this)){
            baseInfoFragment.setNumber(1);
        } else {
            baseInfoFragment.setNumber(0);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_about:
                startActivity(new Intent(this, AboutActivity.class));
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    protected void addLog(String tag, String info) {
        TestModeUtil.addLogString(tag, info);
    }

    /**
     * 切换Fragment
     *
     * @param lastIndex 上个显示Fragment的索引
     * @param index     需要显示的Fragment的索引
     */
    public void switchFragment(int lastIndex, int index) {
        if (lastIndex == index) return;
        FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
        transaction.hide(fragments[lastIndex]);
        if (!fragments[index].isAdded()) {
            transaction.add(R.id.content, fragments[index]);
        }
        transaction.show(fragments[index]).commitAllowingStateLoss();
    }


    /**
     * 主页面中的各个fragment的按钮点击事件都汇集到这里处理，已经在layout xml文件中声明
     */
    public void onButtonClick(View view) {
        if (view instanceof Button) {
            addLog("点击Button", ((Button) view).getText().toString());
        } else if (view instanceof TextView) {
            addLog("点击TextView", ((TextView) view).getText().toString());
        } else {
            addLog("点击控件", view.getClass().getSimpleName());
        }
        try {
            switch (view.getId()) {
                //基本信息页面按钮事件
                case R.id.btn_app_register:
                    HeytapPushManager.register(this, AppParam.appKey, AppParam.appSecret, mPushCallback);//setPushCallback接口也可设置callback
                    break;
                case R.id.btn_app_unregister:
                    HeytapPushManager.unRegister();
                    break;
                case R.id.btn_check_opush_support:
                    addLog("isSupportPush", "" + HeytapPushManager.isSupportPush(this));
                    break;
                case R.id.btn_get_regid:
                    HeytapPushManager.getRegister();
                    break;
                case R.id.btn_open_notification_setting:
                    HeytapPushManager.openNotificationSettings();
                    break;
                case R.id.btn_check_appid:
                    showResult("AppId", AppParam.appId);
                    showResult("TODO", "功能暂未实现");
                    break;
                case R.id.btn_get_push_service_status:
                    HeytapPushManager.getPushStatus();
                    break;
                case R.id.btn_get_statusbar_status:
                    HeytapPushManager.getNotificationStatus();
                    break;
                case R.id.btn_get_version_info:
                    StringBuilder sb = new StringBuilder();
                    sb.append("McsVerCode:").append(HeytapPushManager.getPushVersionCode());
                    sb.append(",McsVerName:").append(HeytapPushManager.getPushVersionName());
                    sb.append(",SdkVer:").append(HeytapPushManager.getSDKVersionCode());
                    addLog("Version", sb.toString());
                    break;
                //推送设置页面信息事件
                case R.id.btn_pause_push:
                    HeytapPushManager.pausePush();
                    break;
                case R.id.btn_resume_push:
                    HeytapPushManager.resumePush();
                    break;
                case R.id.bt_is_support:
                    LogUtil.d("HeytapPushManager",HeytapPushManager.isSupportPush(this)+"");
                    break;
                case R.id.bt_clear_notification:
                    HeytapPushManager.clearNotifications();
                case R.id.bt_static1:
                    HeytapPushManager.statisticMessage(this, new MessageStat(getPackageName(), "TestDemo"));
                case R.id.bt_static2:
                    List<MessageStat> list = new ArrayList<>();
                    list.add(new MessageStat(getPackageName(), "TestDemo"));
                    list.add(new MessageStat(getPackageName(), "TestDemo"));
                    list.add(new MessageStat(getPackageName(), "TestDemo"));
                    HeytapPushManager.statisticMessage(this,list );
                    break;
                case R.id.btn_sta1:
                    HeytapPushManager.statisticEvent(this, EventConstant.EventId.EVENT_ID_PUSH_NO_SHOW,null);
                    HeytapPushManager.statisticEvent(this, EventConstant.EventId.EVENT_ID_READ_MESSAGE,null);
                    break;
                case R.id.btn_sta2:
//                    Map<String,List<DataMessage>> map = new HashMap<>();
//                    DataMessage dataMessage1 = new DataMessage();
//                    dataMessage1.setTaskID("111111");
//                    List<DataMessage> dataMessagesList1 = new ArrayList<>();
//                    dataMessagesList1.add(dataMessage1);
//                    DataMessage dataMessage2 = new DataMessage();
//                    dataMessage1.setTaskID("222222");
//                    List<DataMessage> dataMessagesList2 = new ArrayList<>();
//                    dataMessagesList2.add(dataMessage2);
//                    map.put(EventConstant.EventId.EVENT_ID_READ_MESSAGE,dataMessagesList1);
//                    map.put(EventConstant.EventId.EVENT_ID_PUSH_DELETE,dataMessagesList2);
//                    HeytapPushManager.statisticEvent(this, map);
//                    for(String st : map.keySet()){
//                        List<DataMessage> dataMessageList =map.get(st);
//                        for(DataMessage message : dataMessageList){
//                            LogUtil.d("eventId:"+st+"taskId:"+message.getTaskID());
//                        }
//                    }
                    break;
                case R.id.btn_start_press:
                    flag =true;
                    testData();
                    break;
                case R.id.btn_stop_press:
                    flag =false;
                    break;
                case R.id.red_badge:
                    ConfigManager.getInstant().setRedBadge(true);
                    addLog("红点通道", "red_badge");
                    break;
                case R.id.no_red_badge:
                    ConfigManager.getInstant().setRedBadge(false);
                    addLog("无红点通道", "no_red_badge");
                    break;
                case R.id.clear_red_badge:
                    NotificationUtil.clearRedBadge(this);
                    baseInfoFragment.setNumber(0);
                    break;
                case R.id.btn_switch_enable:
                    HeytapPushManager.enableAppNotificationSwitch(setAppNotificationCallBackService);
                    break;
                case R.id.btn_switch_disable:
                    HeytapPushManager.disableAppNotificationSwitch(setAppNotificationCallBackService);
                    break;
                case R.id.btn_seitch_get:
                    HeytapPushManager.getAppNotificationSwitch(getAppNotificationCallBackService);
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            showResult("Exception", e.getLocalizedMessage());
            TestModeUtil.addLogString(e.getLocalizedMessage());
        }
    }

    /**
     * 修改每周推送时间或者修改每天推送时间之后会触发此事件
     */
    public void onPushTimeConfigChange(@Nullable View view) {
        List<Integer> times = getPushTimesSet();//时间段要符合规则
        List<Integer> days = getPushWeekdays();//用1-7表示一周的7天
        if (days.isEmpty()) {
            showResult("Error", "推送时间设置不能为空！");
            return;
        }
        try {
            HeytapPushManager.setPushTime(days, times.get(0), times.get(1), times.get(2), times.get(3));
            ((PushConfigFragment) pushConfigFragment).onPushDaysChange();
        } catch (Exception e) {
            showResult("Error", e.getLocalizedMessage());
            TestModeUtil.addLogString(e.getLocalizedMessage());
        }
    }

    /**
     * 编辑框中输入的别名或者标签列表设置，使用分号或者空格，逗号等隔开,从fragment中获取
     *
     * @return 至少包含一个参数，否则会报错
     */
    private List<String> getAliasOrTopicsList() {
        String content = ((UserBandingFragment) userBandingFragment).getInputText();
        List<String> list = Arrays.asList(content.split("[,;`~!?\\s.，。；？！·]"));
        LogUtil.d("list:" + Arrays.toString(list.toArray()));
        return list;
    }

    /**
     * 每周可加收推送消息的时间,从fragment中获取
     */
    private List<Integer> getPushWeekdays() {
        return ((PushConfigFragment) pushConfigFragment).getPushWeekdays();
    }

    /**
     * 每天接受推送消息的时间,从fragment中获取
     */
    private List<Integer> getPushTimesSet() {
        return ((PushConfigFragment) pushConfigFragment).getPushTime();
    }

    /**
     * 此方法会将结果进行回显
     */
    private void showResult(@Nullable String tag, @NonNull String msg) {
        addLog(tag, msg);
        LogUtil.d(tag + ":" + msg);
    }


}
